/******************************************************************************
FILE:				filesel.m
LANGUAGE:			Objective-C
SYSTEM:				OpenStep
USER-INTERFACE:		OpenStep
DESCRIPTION

    This program  is intended as a  command line tool to  let the user
    select files or directory  with the NSOpenPanel or the NSSavePanel
    and report them to the calling script.

USAGE
	filesel.app/filesel --help

    filesel.app/filesel [ [-s|--save|-o|--open] 
                          [-f|--files|-nf|--no-files] 
                          [-d|--directories|-nd|--no-directory]
                          [-m|--multiple|-nm|--no-multiple]
                          [-T|--title TITLE]
                          [-P|--prompt PROMPT]
                          [-D|--from-directory DIRECTORY]
                          [-R|--required-file-type TYPE]
                          [-a|--file-packages-as-directories
                           |-na|--no-file-package-as-directory]]
                        [TYPE] ]... [ -|-- [TYPE]... ]

    status: 0 = OK button pressed.
            1 = Cancel button pressed.
            2 = bad option.

    stdout: NL-separated list of absolute file paths.

AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	2001-05-15 <PJB> Creation.
BUGS
    Uses GNU specific get_current_directory().
LEGAL
    GPL
	Copyright Pascal J. Bourguignon 2001 - 2001

    This  program is  free software;  you can  redistribute  it and/or
    modify it  under the  terms of the  GNU General Public  License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program  is distributed in the  hope that it  will be useful,
    but  WITHOUT ANY WARRANTY;  without even  the implied  warranty of
    MERCHANTABILITY or FITNESS FOR  A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a  copy of the GNU General Public License
    along with  this program; see the  file COPYING; if  not, write to
    the Free  Software Foundation, Inc.,  59 Temple Place,  Suite 330,
    Boston, MA 02111-1307 USA
******************************************************************************/

#define _GNU_SOURCE
#include <unistd.h>

#import <AppKit/AppKit.h>
#import <Foundation/Foundation.h>

@interface FileSel:NSObject
{
    unsigned int    save:1,
                    filePackagesAsDirectories:1,
                    canChooseFiles:1,
                    canChooseDirectories:1,
                    allowMultipleSelection:1;
    NSString*       title;
    NSString*       prompt;
    NSString*       fromDirectory;
    NSString*       requiredFileType;
    NSMutableArray* types;
}

    -(id)initWithArgC:(int)argc argV:(char**)argv;
    -(void)processArgC:(int)argc argV:(char**)argv;
	-(void)doFileSel:sender;
@end //FileSel;


@implementation FileSel

    -(id)initWithArgC:(int)argc argV:(char**)argv
	{
		self=[super init];
		if(self!=nil){
			[[NSNotificationCenter defaultCenter] addObserver:self
				selector:@selector(doFileSel:) 
				name:NSApplicationDidFinishLaunchingNotification
				object:nil];
            [self processArgC:argc argV:argv];
		}
		return(self);
	}//init;


    -(NSString*)getArg:(char**)argv  index:(int)index
    {
        char* pname=argv[0];
        if(argv[index]==0){
            fprintf(stderr,"%s: Missing argument after option '%s'.",
                    pname,argv[index-1]);
            exit(2);
        }
        return([[NSString alloc]initWithCString:argv[index]]);
    }//getArg:index:;


    -(void)processArgC:(int)argc argV:(char**)argv
    {
        /*
        filesel [-s|--save|-o|--open] 
                [-f|--files|-nf|--no-files] 
                [-d|--directories|-nd|--no-directory]
                [-m|--multiple|-nm|--no-multiple]
                [-T|--title TITLE]
                [-P|--prompt PROMPT]
                [-D|--from-directory DIRECTORY]
                [-R|--required-file-type TYPE]
                [-a|--file-packages-as-directories
                 |-na|--no-file-package-as-directory]]
                [TYPE]...
        */
        char* pname=argv[0];
        int i;
        BOOL doTypes=NO;

        save=NO;
        filePackagesAsDirectories=NO;
        canChooseFiles=YES;
        canChooseDirectories=NO;
        allowMultipleSelection=NO;
        [title release];title=nil;
        [prompt release];prompt=nil;
        [fromDirectory release];fromDirectory=nil;
        [requiredFileType release];requiredFileType=nil;
        [types release];types=[[NSMutableArray alloc]init];

        for(i=1;i<argc;i++){
            // NSLog(@"Processing %s\n",argv[i]);
            if(doTypes){
                [types addObject:[self getArg:argv index:i]];
            }else if((strcmp(argv[i],"-s")==0)
                     ||(strcmp(argv[i],"--save")==0)){
                save=YES;
            }else if((strcmp(argv[i],"-o")==0)
                     ||(strcmp(argv[i],"--open")==0)){
                save=NO;
            }else if((strcmp(argv[i],"-f")==0)
                     ||(strcmp(argv[i],"--file")==0)
                     ||(strcmp(argv[i],"--files")==0)){
                canChooseFiles=YES;
            }else if((strcmp(argv[i],"-nf")==0)
                     ||(strcmp(argv[i],"--no-file")==0)){
                canChooseFiles=NO;
            }else if((strcmp(argv[i],"-d")==0)
                     ||(strcmp(argv[i],"--directory")==0)
                     ||(strcmp(argv[i],"--directories")==0)){
                canChooseDirectories=YES;
            }else if((strcmp(argv[i],"-nd")==0)
                     ||(strcmp(argv[i],"--no-directory")==0)){
                canChooseDirectories=NO;
            }else if((strcmp(argv[i],"-m")==0)
                     ||(strcmp(argv[i],"--multiple")==0)){
                allowMultipleSelection=YES;
            }else if((strcmp(argv[i],"-nm")==0)
                     ||(strcmp(argv[i],"--no-multiple")==0)){
                allowMultipleSelection=NO;
            }else if((strcmp(argv[i],"-T")==0)
                     ||(strcmp(argv[i],"--title")==0)){
                i++;
                title=[self getArg:argv index:i];
            }else if((strcmp(argv[i],"-P")==0)
                     ||(strcmp(argv[i],"--prompt")==0)){
                i++;
                prompt=[self getArg:argv index:i];
            }else if((strcmp(argv[i],"-D")==0)
                     ||(strcmp(argv[i],"--from-directory")==0)){
                i++;
                fromDirectory=[self getArg:argv index:i];
            }else if((strcmp(argv[i],"-R")==0)
                     ||(strcmp(argv[i],"--required-file-type")==0)){
                i++;
                requiredFileType=[self getArg:argv index:i];
            }else if((strcmp(argv[i],"-a")==0)
                     ||(strcmp(argv[i],"--file-package-as-directory")==0)
                     ||(strcmp(argv[i],"--file-packages-as-directories")==0)){
                filePackagesAsDirectories=YES;
            }else if((strcmp(argv[i],"-na")==0)
                     ||(strcmp(argv[i],"--no-file-package-as-directory")==0)
                     ||(strcmp(argv[i],"--no-file-packages-as-directories")==0)){
                filePackagesAsDirectories=NO;
            }else if((strcmp(argv[i],"-h")==0)
                     ||(strcmp(argv[i],"--help")==0)){
                fprintf(stdout,
                        "filesel [ [-s|--save|-o|--open] \n"
                        "          [-f|--files|-nf|--no-files] \n"
                        "          [-d|--directories|-nd|--no-directory]\n"
                        "          [-m|--multiple|-nm|--no-multiple]\n"
                        "          [-T|--title TITLE]\n"
                        "          [-P|--prompt PROMPT]\n"
                        "          [-D|--from-directory DIRECTORY]\n"
                        "          [-R|--required-file-type TYPE]\n"
                        "          [-a|--file-packages-as-directories"
                        "           |-na|--no-file-package-as-directory]]\n"
                        "          [TYPE] ]... [ -|--  [TYPE]... ]\n");
                exit(0);
            }else if((strcmp(argv[i],"-")==0)
                     ||(strcmp(argv[i],"--")==0)){
                doTypes=YES;
            }else if(argv[i][0]!='-'){
                [types addObject:[self getArg:argv index:i]];
            }else{
                fprintf(stderr,"%s: Invalid  option '%s'.",pname,argv[i]);
                exit(2);
            }
        }
    }//processArgC:argV:;



    -(void)preparePanel:(NSSavePanel*)panel
    {
        if(title!=nil){
            [panel setTitle:title];
            // NSLog(@"title=%@",title);
        }else{
            // NSLog(@"title is nil");
        }
        if(prompt!=nil){
            [panel setPrompt:prompt];
            // NSLog(@"prompt=%@",prompt);
        }else{
            // NSLog(@"prompt is nil");
        }
        if(fromDirectory!=nil){
            if(![fromDirectory isAbsolutePath]){
                [fromDirectory autorelease];
                fromDirectory=[[NSString alloc]initWithFormat:@"%s/%@",
                                               get_current_dir_name(),
                                               fromDirectory];
            }
            [panel setDirectory:fromDirectory];
            // NSLog(@"fromDirectory=%@",fromDirectory);
        }else{
            // NSLog(@"fromDirectory is nil");
        }
        if(requiredFileType!=nil){
            [panel setRequiredFileType:requiredFileType];
            // NSLog(@"requiredFileType=%@",requiredFileType);
        }else{
            // NSLog(@"requiredFileType is nil");
        }
        [panel setTreatsFilePackagesAsDirectories:
                   filePackagesAsDirectories];
        // NSLog(@"filePackagesAsDirectories=%d",filePackagesAsDirectories);
    }//preparePanel;


	-(void)doFileSel:sender
	{
        if(save){
            NSSavePanel* panel=[NSSavePanel savePanel];
            [self preparePanel:panel];
            if([panel runModal]==NSOKButton){
                fprintf(stdout,"%s\n",[[panel filename]cString]);
                exit(0);
            }else{
                exit(1);
            }
        }else{
            NSOpenPanel* panel=[NSOpenPanel openPanel];
            [self preparePanel:panel];
            [panel setCanChooseFiles:canChooseFiles];
            // NSLog(@"canChooseFiles=%d",canChooseFiles);
            [panel setCanChooseDirectories:canChooseDirectories];
            // NSLog(@"canChooseDirectories=%d",canChooseDirectories);
            [panel setAllowsMultipleSelection:allowMultipleSelection];
            // NSLog(@"allowMultipleSelection=%d",allowMultipleSelection);
            if([panel runModalForTypes:types]==NSOKButton){
                NSArray* filenames=[panel filenames];
                int count=[filenames count];
                int i;
                for(i=0;i<count;i++){
                    fprintf(stdout,"%s\n",[[filenames objectAtIndex:i]cString]);
                }
                exit(0);
            }else{
                exit(1);
            }
        }
	}//doFileSel:;

@end // FileSel;




int main(int argc,char** argv)
{
	extern BOOL initialize_gnustep_backend(void);
	NSAutoreleasePool* pool=[NSAutoreleasePool new];
	(void)initialize_gnustep_backend();
	(void)[NSApplication sharedApplication];
	(void)[[[FileSel alloc]initWithArgC:argc argV:argv]autorelease];
	[NSApp run];
	[pool release];
    return(0);
}//main;

/*** filesel.m                        -- 2001-05-15 00:38:24 -- pascal   ***/
